#ifndef __norm_norm_h
#define __norm_norm_h

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <pcap.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define NEW(x)  ((x*)malloc(sizeof(x)))

#ifdef HAVE_LIBMEMPCAP
#  include <mpcap.h>

#  define MPCAP_SCALING      1.1

#  define PCAP_OPEN_OFFLINE              mpcap_open_offline
#  define PCAP_CLOSE                     mpcap_close
#  define PCAP_LOOP                      mpcap_loop
#  define PCAP_DISPATCH                  mpcap_dispatch
#  define PCAP_DUMP_OPEN(pcap, file)     mpcap_dump_open(pcap, MPCAP_SCALING, file)
#  define PCAP_DUMP_CLOSE                mpcap_dump_close
#  define PCAP_DUMP                      mpcap_dump
#else 
#  define PCAP_OPEN_OFFLINE  pcap_open_offline
#  define PCAP_CLOSE         pcap_close
#  define PCAP_LOOP          pcap_loop
#  define PCAP_DISPATCH      pcap_dispatch
#  define PCAP_DUMP_OPEN     pcap_dump_open
#  define PCAP_DUMP_CLOSE    pcap_dump_close
#  define PCAP_DUMP          pcap_dump
#endif

#ifndef FALSE
#define FALSE   0
#define TRUE    1
#endif

/* Linux uses these defines in netinet/ip.h and netinet/tcp.h to
   use the correct struct ip and struct tcphdr */
#ifndef __FAVOR_BSD
#define __FAVOR_BSD
#endif
#ifndef __USE_BSD
#define __USE_BSD
#endif
#ifndef __BSD_SOURCE
#define __BSD_SOURCE
#endif

#define NUM_IFS          2

struct snap_header
{
  u_char dsap;
  u_char ssap;
  u_char ctrl;
  u_char snap[5];
};



typedef struct _interface
{
  pcap_t             *pcap;   /* pcap handler */
  pcap_dumper_t      *pcap_dumper;
  char               *dev;    /* ASCII device name */
  int                 s;      /* raw socket */
  char               *addrs;  /* IP address string */
  int                 dlt;    /* data link type */
  struct sockaddr_in  ifip;   /* IPv4 address structure */
}
Interface;


typedef void(*NormFunc) (void *data);
typedef int(*NormCmpFunc) (void *d1, void *d2);

extern Interface nifs[NUM_IFS];
extern int       sock;

#endif
