#ifndef __norm_options_h
#define __norm_options_h

#include <norm.h>

#define   FATAL_LOGLEVEL  1
#define    NORM_LOGLEVEL  3
#define    INFO_LOGLEVEL  5
#define WARNING_LOGLEVEL  7
#define   NOISE_LOGLEVEL 10
#define NESTING_LOGLEVEL 50

#define print_ether(d, x)   if (options.debuglevel >= (d) && options.show_ether) { printf("ETH:   "); printf x; }
#define print_fddi(d, x)    if (options.debuglevel >= (d) && options.show_fddi)  { printf("FDDI:  "); printf x; }
#define print_ip(d, x)      if (options.debuglevel >= (d) && options.show_ip)    { printf("IP:    "); printf x; }
#define print_icmp(d, x)    if (options.debuglevel >= (d) && options.show_icmp)  { printf("ICMP:  "); printf x; }
#define print_tcp(d, x)     if (options.debuglevel >= (d) && options.show_tcp)   { printf("TCP:   "); printf x; }
#define print_udp(d, x)     if (options.debuglevel >= (d) && options.show_udp)   { printf("UDP:   "); printf x; }
#define print_icmp(d, x)    if (options.debuglevel >= (d) && options.show_icmp)  { printf("ICMP:  "); printf x; }

#define print_ether_(d, x)  if (options.debuglevel >= (d) && options.show_ether) { printf x; }
#define print_fddi_(d, x)   if (options.debuglevel >= (d) && options.show_fddi)  { printf x; }
#define print_ip_(d, x)     if (options.debuglevel >= (d) && options.show_ip)    { printf x; }
#define print_icmp_(d, x)   if (options.debuglevel >= (d) && options.show_icmp)  { printf x; }
#define print_tcp_(d, x)    if (options.debuglevel >= (d) && options.show_tcp)   { printf x; }
#define print_udp_(d, x)    if (options.debuglevel >= (d) && options.show_udp)   { printf x; }
#define print_icmp_(d, x)   if (options.debuglevel >= (d) && options.show_icmp)  { printf x; }

#define NUM_TRUSTED      10
#define NUM_SWITCHES     31
#define SWITCH_TYPE      int
#define FIRST_SWITCH     options.passive

typedef struct _log_level
{
  int      level;
  char *   id;
}
LogLevel;

typedef struct _norm_options
{
  /* Miscellaneous. */

  int             debuglevel;
  int             verbose;
  int             show_code_location;  /* Whether to print source file & line number */
  char           *configfile;

  /* Trusted side configuration */

  int             num_trusted;                 /* Number of trusted networks */
  struct in_addr  trusted_net[NUM_TRUSTED];    /* Array of trusted network's IPs */
  struct in_addr  trusted_mask[NUM_TRUSTED];   /* Array of trusted network's IP masks */

  char           *in_tracefile;                /* Trace logfile for reading */
  char           *out_tracefile;                /* Trace logfile for writing */

  /* Interface configuration */

  int             num_ifs;                     /* Number of interfaces */
  char           *ifs[NUM_IFS];                /* Interface names. */

  /* Enable/Disable all checks: */

  int             passive;


  /* Network level output switches: */

  int             show_ether;
  int             show_fddi;
  int             show_ip;
  int             show_tcp;
  int             show_udp;
  int             show_icmp;

  /* Normalization switches: */

  int             ip_drop_fragments_with_df;
  int             ip_clear_df_and_offset;
  int             ip_check_fragment_offset;
  int             ip_check_options;
  int             ip_header_length_check;
  int             ip_total_length_check;
  int             ip_header_checksum_check;
  int             ip_header_addr_check;
  int             ip_header_clear_tos;
  int             ip_check_ttl;
  int             ip_ttl_value;
  int             ip_clear_rf;

  int             tcp_need_syn;
  int             tcp_verify_checksum;
  int             tcp_check_flags;
  int             tcp_check_lengths;
  int             tcp_check_options;
  int             tcp_clear_reserved;

  int             icmp_drop_nonzero_code;
  int             icmp_drop_broadcast;
  int             icmp_drop_multicast;
  int             icmp_verify_checksum;
  int             icmp_drop_sourcequench;

  int             udp_verify_checksum;
  int             udp_verify_length;
}
NormOptions;

extern NormOptions options;

void   norm_options_read(int argc, char** argv);
void   norm_options_install_sighandler(void);
char * norm_options_get_dbg_id(int loglevel);

#endif
