#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <errno.h>

#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <fcntl.h>
#include <string.h>
#include <arpa/inet.h>

#include <norm.h>
#include <norm.h>
#include <norm_debug.h>
#include <norm_ether.h>
#include <norm_fddi.h>
#include <norm_ip_fragment.h>
#include <norm_debug.h>
#include <norm_packet.h>


u_int
norm_packet_get_length_link(NormPacket *p)
{
  return (p->length);
}

u_int   
norm_packet_get_length_ip(NormPacket *p)
{
  if (!p->network_data)
    return (0);

  return ntohs(((struct ip*)(p->network_data))->ip_len);
}


void
norm_packet_create(NormPacket *p, int length_needed, Interface *nif)
{
  int           padding;

  if (!p)
    return;

  norm_empty_packet(p);
  padding = 0;

  switch (pcap_datalink(nif->pcap))
    {
    case DLT_EN10MB:
      padding = ETHER_HDR_LEN;
      break;
    case DLT_FDDI:
      padding = FDDI_HDR_LEN + SNAP_LEN;
      break;
    default:
      padding = 0;
    }
  
  p->link_data      = malloc(padding + length_needed);
  p->network_data   = p->link_data + padding; 
  p->length         = padding + length_needed;
  p->allocated_data = TRUE;

  bzero(p->link_data, padding + length_needed);

  switch (pcap_datalink(nif->pcap))
    {
    case DLT_EN10MB:
      {
	/* Fill in dummy Ethernet II header. */
	struct ether_header *eh;

	D(NOISE_LOGLEVEL, ("Allocating Ethernet packet.\n"));
	
	eh = (struct ether_header*)p->link_data;
	eh->ether_type = htons(ETHERTYPE_IP);
      }
      break;
    case DLT_FDDI:
      {
	/* Fill in a dummy IP-carrying SNAP header. */
	struct snap_header *sh;

	D(NOISE_LOGLEVEL, ("Allocating FDDI packet.\n"));

	sh = (struct snap_header*)p->link_data + FDDI_HDR_LEN;
	sh->dsap = sh->ssap = 0xaa;
	sh->ctrl = 0;
	sh->snap[0] = sh->snap[1] = sh->snap[2]
	  = sh->snap[4] = 0;
	sh->snap[3] = 8;
      }
      break;
    default:
      padding = 0;
    }

}


void
norm_empty_packet(NormPacket *p) 
{
  if (p)
    {
      if (p->allocated_data)
	{
	  if (p->link_data)
	    free (p->link_data);
	  else if (p->network_data)
	    free (p->network_data);
	  else if (p->transport_data)
	    free (p->transport_data);      
	  else if (p->app_data)
	    free (p->app_data);	  
	}

      bzero(p, sizeof(NormPacket));
    }
}
