#ifndef __norm_packet_h
#define __norm_packet_h

#include <norm.h>

typedef enum protocol_layer
{
  DATALINK,
  NETWORK,
  TRANSPORT,
  APPLICATION
}
NormLayer;

typedef struct norm_packet
{
  struct pcap_pkthdr   *ph;
  struct timeval             *timestamp;
  int                         length;

  u_char                     *link_data;
  u_char                     *network_data;
  u_char                     *transport_data;
  u_char                     *app_data;

  int                         network_data_recalc_checksum;
  int                         transport_data_recalc_checksum;
  int                         allocated_data;
}
NormPacket;


u_int         norm_packet_get_length_link(NormPacket *p);
u_int         norm_packet_get_length_ip(NormPacket *p);

/* Fills in NormPacket with enough data to hold a packet of
   length_needed bytes starting at the network layer. The
   number of bytes needed for link-level encapsulation for the
   given interface is added internally.
*/
void          norm_packet_create(NormPacket *p, int length_needed, Interface *nif);

/* Frees the data portion of a NormPacket -- not the other pointers. */
void          norm_empty_packet(NormPacket *p);


#endif
